#ifndef VNET_YARP_DETAILS_HH_
#define VNET_YARP_DETAILS_HH_

#include <iostream>
#include <string>
#include <yarp/os/all.h>

namespace vnet {
	namespace yarp {
		template <typename T>
		class Reader {
			private:
				::yarp::os::Port port;
				::yarp::os::PortReaderBuffer<T> reader;

			public:
				Reader(const std::string& uri) : port() {
					port.open(uri.c_str());
					reader.attach(port);
				}

				bool read(T& value, bool wait) {
					T* res = reader.read(wait);
					if (res == NULL)
						return false;

					value = *res;
					return true;
				}
		};

		template <typename T>
		class Writer {
			private:
				T value;
				::yarp::os::Port port;

			public:
				Writer(const std::string& local_uri, const std::string& remote_uri) 
				{
					port.open(local_uri.c_str());
					port.addOutput("/vnet/in");

					::yarp::os::Bottle env;
					env.addString(local_uri.c_str());
					env.addString(remote_uri.c_str());
					port.setEnvelope(env);
				}

				T& prepare() { return value; }

				bool write() { 
					::yarp::os::Bottle success;
					port.write(value, success); 

					int succ = success.get(0).asInt();
					return succ;
				}
		};
	}
}

#endif /* VNET_YARP_DETAILS_HH_ */
