#ifndef VNET_YARP_FILTER_HH_
#define VNET_YARP_FILTER_HH_

#include <utility>
#include <string>

#include <boost/functional/factory.hpp>
#include <boost/program_options.hpp>

namespace vnet {
	namespace yarp {
		class filter_base {
			public:
				virtual bool filter(const std::pair<std::string, std::string>& p) = 0;
				virtual std::string repr() const = 0;
				virtual ~filter_base() {}
		};

		class filter_factory {
			typedef boost::function<filter_base* (const std::vector<std::string> &)> FilterCtor;
			std::map<std::string, FilterCtor> factory_;

			public:
				filter_factory();
				filter_base* make_filter(const std::string&, const std::vector<std::string>&);

				template <class Filter>
				bool register_filter(const std::string& name) 
				{
					if (factory_.find (name) != factory_.end ())
						return false;
					else
						factory_[name] = boost::factory<Filter*>();

				}
		};
	}
}

#endif /* VNET_YARP_FILTER_HH_ */
