#ifndef VNET_YARP_MORSE_FILTER_HH_
#define VNET_YARP_MORSE_FILTER_HH_ 

#include <vector>
#include <string>

#include <vnet_filter.hh>
#include <vnet_morse.hh>


namespace vnet {
	namespace yarp {

		class filter_morse : public filter_base {
			private:

			public:
				filter_morse()
				{};

				filter_morse(const std::vector<std::string>& argv)
				{
					po::options_description desc("Allowed options");
					desc.add_options()
						("threshold", po::value<double>()->default_value(20.0), "threshold")
					;

					po::variables_map vm;
					po::store(po::command_line_parser(argv).
							options(desc).run(), vm);
					po::notify(vm);

					distance_threshold = vm["threshold"].as<double>();
				}


				bool filter(const std::pair<std::string, std::string>& p)
				{
					comm_status status = 
					morse_world::get_instance()->compute_comm_status(p.first, p.second);
					if (status.success == false or status.has_visibility == false)
						return false;

					return (status.distance < distance_threshold);
				}

				std::string repr() const
				{
					std::ostringstream oss;
					oss << "morse (threshold: " << distance_threshold << ")";
					return oss.str();
				}


			private:
				double distance_threshold;

		};
	}
}

#endif /* VNET_YARP_MORSE_FILTER_HH_ */
