#ifndef VNET_YARP_RANDOM_FILTER_HH_
#define VNET_YARP_RANDOM_FILTER_HH_ 

#include <vector>
#include <string>

#include <boost/random.hpp>
#include <boost/program_options.hpp>

#include <vnet_filter.hh>


namespace po = boost::program_options;

namespace vnet {
	namespace yarp {

		class filter_random : public filter_base {
			public:
				filter_random() :
					generator_ (),
					uniform_dist_ (0.0, 1.0),
					rnd_ (generator_, uniform_dist_),
					threshold_(0.5)
				{};

				filter_random(const std::vector<std::string>& argv):
					generator_ (),
					uniform_dist_ (0.0, 1.0),
					rnd_ (generator_, uniform_dist_)
				{
					po::options_description desc("Allowed options");
					desc.add_options()
						("threshold", po::value<double>()->default_value(0.5), "threshold")
					;

					po::variables_map vm;
					po::store(po::command_line_parser(argv).
							options(desc).run(), vm);
					po::notify(vm);

					threshold_ = vm["threshold"].as<double>();
				};

				bool filter(const std::pair<std::string, std::string>& )
				{
					return (rnd_() > threshold_);
				}

				std::string repr() const
				{
					std::ostringstream oss;
					oss << "random (threshold: " << threshold_ << ")";
					return oss.str();
				}

			private:
				boost::mt19937 generator_;
				boost::uniform_real<> uniform_dist_;
				boost::variate_generator<boost::mt19937 &, boost::uniform_real<> > rnd_;
				double threshold_;
		};
	}
}

#endif /* VNET_YARP_RANDOM_FILTER_HH_ */
