#include "vnet_morse.hh"

using namespace vnet;

morse_world::morse_world(const std::string& hostname, const std::string& port, bool delay):
	socket(io_service), counter(0), hostname(hostname), port(port), connected(false)
{
	if (delay) 
		return;

	init();
}

void
morse_world::init()
{
	using namespace boost::asio::ip;
	tcp::resolver resolver(io_service);
	tcp::resolver::query query(hostname, port);

	tcp::resolver::iterator endpoint_iterator = resolver.resolve(query);
	tcp::resolver::iterator end;

	boost::system::error_code error = boost::asio::error::host_not_found; 
	while (error && endpoint_iterator != end)
	{
		socket.close();
		socket.connect(*endpoint_iterator++, error);
	}

	if (error)
		throw boost::system::system_error(error);

	connected = true;
}

std::string morse_world::rpc(const std::string& s)
{
	if (!connected) init();

	boost::asio::streambuf request;
	std::ostream request_stream(&request);
	request_stream << s;
	boost::asio::write(socket, request);

	boost::asio::streambuf response;
	boost::asio::read_until(socket, response, "\n");

	return std::string(std::istreambuf_iterator<char>(&response),
					   std::istreambuf_iterator<char>());
}

comm_status morse_world::compute_comm_status(const std::string& robot1,
											 const std::string& robot2)
{
	std::ostringstream oss;
	oss << "req" << counter++ << " communication distance_and_view ";
	oss << "['" << robot1 << "', '" << robot2 << "']\n";

	std::string ans = rpc(oss.str());
	std::istringstream iss(ans);
	std::string req, succ;
	comm_status result;

	iss >> req >> succ;

	result.success = (succ == "SUCCESS");
	if (!result.success)
		return result;

	std::string next;
	iss >> next;
	next.erase(next.size() - 1, 1);
	next.erase(0, 1);
	std::istringstream iss2(next);
	iss2 >> result.distance;

	iss >> next;
	next.erase(next.size() - 1, 1);
	result.has_visibility = (next == "True");

	return result;
}

static morse_world* our_private_instance = 0;

morse_world* morse_world::get_instance() { return our_private_instance; }

morse_world* morse_world::init_instance(const std::string& hostname, const std::string& port) 
{ 
	our_private_instance = new morse_world(hostname, port, true);
	return our_private_instance;
}
