#ifndef VNET_MORSE_HH_
#define VNET_MORSE_HH_

#include <string>

#include <boost/asio.hpp>
#include <boost/utility.hpp>

namespace vnet {

	struct comm_status {
		bool success;
		double distance;
		bool has_visibility;
	};

	class morse_world : public boost::noncopyable {
		public:
			static morse_world* get_instance();
			static morse_world* init_instance(const std::string& hostname,
											  const std::string& port);

			comm_status compute_comm_status(const std::string& robot1, 
											const std::string& robot2);

			morse_world(const std::string& hostname, const std::string& port, bool delay = false);
		private:
			boost::asio::io_service io_service;
			boost::asio::ip::tcp::socket socket;
			size_t counter;
			std::string hostname;
			std::string port;
			bool connected;

			std::string rpc(const std::string& s);
			void init();
	};
}

#endif /* VNET_MORSE_HH_ */
