#include <cassert>
#include <iostream>

#include <vnet_filter.hh>
#include <vnet_table.hh>
#include <vnet_proxy.hh>

#include <boost/thread/thread.hpp>

bool terminated;

void vnet_main(vnet::yarp::vnetProxy& proxy)
{
	while (!terminated) { proxy.run(); }
}

#define USE_VNET
#include <vnet_yarp.hh>

typedef vnet::yarp::Reader<yarp::os::Bottle> ReaderT;
typedef vnet::yarp::Writer<yarp::os::Bottle> WriterT;
typedef boost::lock_guard<vnet::yarp::table> lock_table;

void success_send(WriterT& writer, ReaderT& reader, int x, double y, const std::string& s)
{
	yarp::os::Bottle& bottle = writer.prepare();
	bottle.clear();
	bottle.addInt(x);
	bottle.addDouble(y);
	bottle.addString(s.c_str());
	assert(writer.write());

	yarp::os::Bottle b;
	assert(reader.read(b, true));
	assert(b.get(0).asInt() == x);
	assert(b.get(1).asDouble() == y);
	std::string s2(b.get(2).asString().c_str());
	assert(s2 == s);
}

void fail_send(WriterT& writer, int x, double y, const std::string& s)
{
	yarp::os::Bottle& bottle = writer.prepare();
	bottle.clear();
	bottle.addInt(x);
	bottle.addDouble(y);
	bottle.addString(s.c_str());
	assert(!writer.write());
}

int main(int argc, char ** argv)
{
	yarp::os::Network::init();

	vnet::yarp::filter_factory factory;
	vnet::yarp::filter_base* filter = factory.make_filter("pass", std::vector<std::string>());
	vnet::yarp::table mtable(filter);
	vnet::yarp::vnetProxy proxy(mtable);

	boost::thread thr(boost::bind(&vnet_main, boost::ref(proxy)));

	WriterT mana_out("/mana/out", "/minnie/in");
	WriterT ressac_out("/ressac/out", "/minnie/in");
	ReaderT minnie_in("/minnie/in");

	success_send(mana_out, minnie_in, 10, 20.0, "pipo");
	success_send(ressac_out, minnie_in, 20, 40.0, "pipo2");

	{
		lock_table lk(mtable);
		mtable.add("mana", "minnie", factory.make_filter("pass", std::vector<std::string>()));
	}

	success_send(mana_out, minnie_in, 35, 42.0, "foo");
	success_send(ressac_out, minnie_in, 35, 84.0, "foo2");


	{
		lock_table lk(mtable);
		mtable.add("mana", "minnie", factory.make_filter("block", std::vector<std::string>()));
	}

	fail_send(mana_out, 200, 3.14, "e");
	success_send(ressac_out, minnie_in, 200, 3.40, "e2");

	{
		lock_table lk(mtable);
		mtable.remove("mana", "minnie", 1);
	}

	success_send(mana_out, minnie_in, 50, 73.3, "bar");
	success_send(ressac_out, minnie_in, 100, 152.0, "bar2");

	{
		lock_table lk(mtable);
		mtable.add("mana", "minnie", factory.make_filter("block", std::vector<std::string>()));
	}

	fail_send(mana_out, 1000, 1.22, "pi");
	success_send(ressac_out, minnie_in, 10000, 52.0, "pi2");

	{
		lock_table lk(mtable);
		mtable.clear("mana", "minnie");
	}

	success_send(mana_out, minnie_in, 100, 7.3, "done");

	// set terminated to true, and send a request to make sure the thread terminate
	terminated = true;
	success_send(mana_out, minnie_in, 100, 7.3, "done");
	thr.join();

	yarp::os::Network::fini();
}
